	list p=12f615
	radix dec
#include "p12f615.inc"

	__config _INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _IOSCFS4 & _BOR_OFF

	org 0x0000       ;Reset vector
	goto main

	org 0x0004       ;Interrupt vector
	retfie

;=============================================================
; Constants
;=============================================================
BUTTON_1_PIN  equ 2
BUTTON_2_PIN  equ 4
BOOTSTRAP_PIN equ 2
RF_DATA_PIN   equ 5

BTN2_NONE     equ 0
BTN2_DOT      equ 1
BTN2_DASH     equ 2

RF_POWER_ON   equ 0
RF_POWER_OFF  equ 1
RF_VGA1       equ 2
RF_VGA2       equ 3

;=============================================================
; Global variables
;=============================================================
rf_code equ 0x7A     ; 0=power on, 1=power off, 2=vga1, 3=vga2
temp    equ 0x7B     ; Temporary 16-bit field
counter equ 0x7D     ; 24-bit counter for delay loops and timing

	org 0x0005

main:

;=============================================================
; I. Initialize 
;     - Initialize I/O pins:
;         GP2: Input
;         GP4: Input
;         GP5: Output, low
;=============================================================
	clrf  GPIO           ; Set all output ports off

	bsf   STATUS, RP0    ; Select bank 1

	clrf  ANSEL          ; Set all pins to digital I/O

	movlw 0x1F
	movwf TRISIO         ; Set GP5 to output mode

	bcf   STATUS, RP0     ; Select bank 0

;=============================================================
; II. Detect desired function from button press
;     No button   = nothing
;     Button 1 .  = nothing (dictated by bootstrap circuit)
;     Button 1 -  = Power on
;     Button 2 .  = Source: VGA1
;     Button 2 -  = Power off
;     Button 2 .. = Source: VGA2
;     Button 2 ._ = Power off
;
;      - If button 2 is pressed:
;         * Reconfigure GP2 as output and Enable bootstrap
;         * If dash:
;            o POWER OFF
;         * Else (dot):
;            o If dot again:
;               * SOURCE: VGA2
;            o Else if dash after dot:
;               * POWER OFF
;            o Else (no activation)
;               * Source: VGA1
;      - Else if button 1 is pressed:
;         * Delay 1 second (if dot, will sleep automatically)
;         * Reconfigure GP2 as output and Enable bootstrap
;         * POWER ON
;      - Else (transient or PICkit2 programmer supplying power):
;         * Sleep
; 
;=============================================================

button_detect:
	btfss GPIO,BUTTON_2_PIN
	goto  button_detect_not_btn2       ; Test for button 2

button_detect_btn2:
	bsf   STATUS, RP0             ; Select bank 1
	bcf   TRISIO, BOOTSTRAP_PIN   ; Make GP2 an output
	bcf   STATUS, RP0             ; Select bank 0
    bsf   GPIO,   BOOTSTRAP_PIN   ; Enable the bootstrap (keep us alive if the button is released)

    call  get_press               ; Get the length of keypress for button 2
    xorlw 1                       ; Test for retval=1
    btfsc STATUS, Z
    goto  button_detect_btn2_dot
    xorlw 3                       ; Test for retval=2 (now 3 because of prior xor)
    btfss STATUS, Z
    goto  shutdown                ; No actual keypress?  Shut down.

button_detect_btn2_dash: ;-
	movlw RF_POWER_OFF
    goto  send_code               ; Send the POWER OFF code

button_detect_btn2_dot:
	call get_press                ; Get the length of the second keypress for button 2
    xorlw 1                       ; Test for retval=1
    btfsc STATUS, Z
    goto button_detect_btn2_dot_dot
    xorlw 3                       ; Test for retval=2 (now 3 because of prior xor)
    btfsc STATUS, Z
    goto button_detect_btn2_dash

button_detect_btn2_dot_only: ;-
    movlw RF_VGA1
    goto  send_code               ; Send the VGA1 code

button_detect_btn2_dot_dot:
    movlw RF_VGA2
    goto  send_code               ; Send the VGA2 code

button_detect_not_btn2:
	btfss GPIO,BUTTON_1_PIN
	goto shutdown                 ; Shutdown if no button is pressed

button_detect_btn1: ;-
	movlw 0xAE
    movwf counter
	movlw 0x13
	movwf (counter+1)
	movlw 0x06
	movwf (counter+2)
button_detect_delay_loop:
    decfsz counter, F
    goto button_detect_delay_loop
	decfsz (counter+1), F
	goto button_detect_delay_loop
	decfsz (counter+2), F
	goto button_detect_delay_loop ; Delay for 1 second

;For initial counter value 06 13 AE, total delay value = 1000001 cycles, including initialization

                                  ; Assume button is still pressed (we're still alive)

	bsf   STATUS, RP0             ; Select bank 1
	bcf   TRISIO, BOOTSTRAP_PIN   ; Make GP2 an output
	bcf   STATUS, RP0             ; Select bank 0
    bsf   GPIO,   BOOTSTRAP_PIN   ; Enable the bootstrap (keep us alive if the button is released)

    movlw RF_POWER_ON
	;goto send_code

;=============================================================
; III. Send the remote code
;       - Send the initial 144ms preamble (send 18 zero bytes)
;       - wait the 5ms dead-air time
;       - Send the 4 bytes, Manchester-encoded
;=============================================================
send_code:
	movwf  rf_code
	movlw  18
	movwf  (temp+1)				; Get ready to send 18 zero bytes
send_preamble:
	clrw
	call   send_byte
	decfsz (temp+1), F
	goto send_preamble

	movlw 0x7A
	movwf counter
	movlw 7
	movwf (counter+1)
send_delay:
	decfsz counter, F
	goto send_delay
	decfsz (counter+1), F
	goto send_delay

	movlw 0x76
	call send_byte
	movlw 0x41
	call send_byte
	movlw 0x79
	call send_byte
	movlw 0x08
	iorwf rf_code, W
	call send_byte

;=============================================================
; IV. Shut down
;       - Tie the data output low
;       - Disable the bootstrap output
;       - Go to sleep
;=============================================================

shutdown:
	clrf  GPIO
	bsf   STATUS, RP0
	movlw 0x1F
	movwf TRISIO
	bcf   STATUS, RP0
	sleep	             ; Go to sleep until power-on reset or brown-out reset
	goto 0

;=============================================================
; get_press:
;  Samples button 2 for press, and returns result in W.
;  Filters out spaces of less than 0.15s
;  Waits for a maximum of 1 second before returning
;  Return value:
;   - If no press within 0.075s, returns 0
;   - If press lasts less than 1s, return 1
;   - If press lasts 1s or longer, return 2
;=============================================================
get_press:
   clrf   rf_code
   bsf    rf_code,7
   btfsc  GPIO, BUTTON_2_PIN

   movlw  0x51                         ;Wait for off state with 1.0s timeout
   movwf  counter
   movlw  0x87
   movwf  (counter+1)
   movlw  0x2
   movwf  (counter+2)
   movlw  0xFF
   movwf  temp
   movlw  37
   movwf  (temp+1)

get_press_wait_for_off:
   btfsc  GPIO, BUTTON_2_PIN
   goto   get_press_still_on           ; Detect if pin is off or on
   decfsz temp, F                      ; (Filter out drops of up to 75ms)
   goto   get_press_filter
   decfsz temp+1, f
   goto   get_press_filter
   btfsc  rf_code,7
   retlw  0                            ; Return 0 if never pressed
   retlw  1                            ; Return 1 : temp depleted, stable off state reached

get_press_still_on:
   bcf   rf_code,7
   movlw 0xFF                          ; 3
   movwf temp                          ; 4 Reset temp
   movlw 37
   movwf (temp+1)

get_press_filter:
   decfsz counter, F                   ; 5
   goto   get_press_wait_for_off       ; 6 (8 cycles per iteration)
   decfsz (counter+1), F
   goto   get_press_wait_for_off
   decfsz (counter+2), F
   goto   get_press_wait_for_off
   retlw  2                            ; Timeout - return 2

;=============================================================
; send_byte:
;  Send a Manchester-encoded byte of data in 8ms
;  Argument: W = byte to encode
;=============================================================
send_byte:
	movwf temp
	call  send_bit
    rlf   temp, F
	call  send_bit
    rlf   temp, F
	call  send_bit
    rlf   temp, F
	call  send_bit
    rlf   temp, F
	call  send_bit
    rlf   temp, F
	call  send_bit
    rlf   temp, F
	call  send_bit
    rlf   temp, F
	call  send_bit
	return

send_bit:
	btfss temp, 7
    goto  send_bit_0
	bcf   GPIO, RF_DATA_PIN
	goto  send_bit_1
send_bit_0
	bsf   GPIO, RF_DATA_PIN
	call  delay_half_ms
	bcf   GPIO, RF_DATA_PIN
	call  delay_half_ms
	return
send_bit_1:
	bcf   GPIO, RF_DATA_PIN
	call  delay_half_ms
	bsf   GPIO, RF_DATA_PIN
	call  delay_half_ms
	return

delay_half_ms:
    movlw 163
	movwf counter
    nop
delay_half_ms_loop:
	decfsz counter, F
	goto delay_half_ms_loop
    return

	end
